# =====================================
# generator=datazen
# version=3.1.4
# hash=6f05f5ec8ef6e670482d6a0966b7641f
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "An interface generator for distributed computing."
PKG_NAME = "ifgen"
VERSION = "3.1.5"
